# SPDX-FileCopyrightText: Copyright © 2023 Idiap Research Institute <contact@idiap.ch>
#
# SPDX-License-Identifier: GPL-3.0-or-later
"""CNN for detecting radiological findings.

A Densenet-121 model for radiological sign detection, using the NIH
CXR-14 label format (ie. 14 outputs).
"""

from torch.nn import BCEWithLogitsLoss
from torch.optim import Adam

from ptbench.data.augmentations import ElasticDeformation
from ptbench.models.densenet import Densenet

model = Densenet(
    train_loss=BCEWithLogitsLoss(),
    validation_loss=BCEWithLogitsLoss(),
    optimizer_type=Adam,
    optimizer_arguments=dict(lr=0.0001),
    augmentation_transforms=[ElasticDeformation(p=0.8)],
    pretrained=False,
    num_classes=14,  # number of classes in NIH CXR-14
)
