# SPDX-FileCopyrightText: Copyright © 2023 Idiap Research Institute <contact@idiap.ch>
#
# SPDX-License-Identifier: GPL-3.0-or-later
"""CNN for Tuberculosis Detection.

Implementation of the model architecture proposed by F. Pasa in the article
"Efficient Deep Network Architectures for Fast Chest X-Ray Tuberculosis
Screening and Visualization".

Reference: [PASA-2019]_
"""

from torch.nn import BCEWithLogitsLoss
from torch.optim import Adam

from ptbench.data.augmentations import ElasticDeformation
from ptbench.models.pasa import Pasa

model = Pasa(
    train_loss=BCEWithLogitsLoss(),
    validation_loss=BCEWithLogitsLoss(),
    optimizer_type=Adam,
    optimizer_arguments=dict(lr=8e-5),
    augmentation_transforms=[ElasticDeformation(p=0.8)],
)
